
// Atxmega128A1 und USARTE0 (PE2 Rx) und (PE3 TX)  115200 bps
// ADCCH0 PINA0 poitiv, GND negativ  
// Intern 1 Volt 
// unsigned Betrieb 0 - 4095 Samples
// keine  Offset
// zeigt bei 0,8 Volt am ADCCH0 +0.8 Volt an !!


#include <avr/pgmspace.h>
#include <string.h>
#include "avr_compiler.h"
#include "clksys_driver.h" 	// Atmel Treiber
#include "adc_driver.h" 	// Atmel Treiber
#include "uart.h"

#include "xitoa.h"

#define NBR_OF_SAMPLES 10 //liest 10 mal den ADC und bildet den Mittelwert

unsigned int get_adValue(void); // ADC init
void run_adc(void);				// ADC run

int main(void)
{
	
	
	// 32Mhz kalibriert Interner Takt (kein Quarz)
//-----------------------------------------------------------------------------
	OSC.CTRL = OSC_RC32MEN_bm | OSC_RC32KEN_bm;
	while(!(OSC.STATUS & OSC_RC32MRDY_bm));
	while(!(OSC.STATUS & OSC_RC32KRDY_bm));
	/* Start Calibrating the 32MHz Clock using * the internal calibrated 32kHz Oscillator */
 	OSC.DFLLCTRL = (0 << OSC_RC32MCREF_bp);
 	DFLLRC32M.CTRL = DFLL_ENABLE_bm;
 	CCP = CCP_IOREG_gc;
	CLK.CTRL = CLK_SCLKSEL_RC32M_gc;
	CLKSYS_Prescalers_Config( CLK_PSADIV_2_gc, CLK_PSBCDIV_1_1_gc ); //16 Mhz 

//-----------------------------------------------------------------------------

	//USART USARTE0 Port konf.
  	PORTE.DIRSET = PIN3_bm;  // Pin33 von PORTE3 (TXD0) ist Ausgang                          
  	PORTE.DIRCLR = PIN2_bm;  // Pin32 von PORTE2 (RXD0) ist Eingang                          
    
    // Init USART 115200bps  ->     
    USARTE0.BAUDCTRLA = 0xD7;   // (F_CPU 16000000 0xD7/0x93),(F_CPU 32000000  0x17/A4)
    USARTE0.BAUDCTRLB = 0x93;   
    USARTE0.CTRLA = 0x10;       // RX is low level interrupt	 
    USARTE0.CTRLC = 0x03;       // Async, No Parity, 1 stop bit, 8 data bits
    USARTE0.CTRLB = 0x18;       // Enable RX and TX // kein CLK2X  !!!!!
             // Initialize Send and Receive buffers

	// alle Interupt einschalten
	PMIC.CTRL |= PMIC_HILVLEN_bm; 
	PMIC.CTRL |= PMIC_MEDLVLEN_bm;
	PMIC.CTRL |= PMIC_LOLVLEN_bm;
	
	PORTB.DIRTGL   |= PIN0_bm;  	// Toggle LED nach ADC-Wandlung

	sei(); 
	
	uart_init();       
	xfunc_out = (void (*)(char))uart_put; // xfunc Ausgabe  mit usart verbinden	

	
	xprintf(PSTR("ADC_Test.. \r")); //Testausgabe

	/*  Select internal 1.0V as reference */ 
	ADCA.REFCTRL = (ADCA.REFCTRL & ~ADC_REFSEL_gm) | ADC_REFSEL_INT1V_gc; 

	/* Single-ended positive input signal */ 
	ADCA.CH0.CTRL = (ADCA.CH0.CTRL & ~ADC_CH_INPUTMODE_gm) | ADC_CH_INPUTMODE_SINGLEENDED_gc; 

	/* Select pin0 */ 
	ADCA.CH0.MUXCTRL = ADC_CH_MUXPOS_PIN0_gc; 

	/* Enable the ADC on port A*/    
	ADCA.CTRLA |= ADC_ENABLE_bm; 

	/* Define the ADC clock prescaler to 16 (0,5 MHz) */ 
	ADCA.PRESCALER = (ADCA.PRESCALER & ~ADC_PRESCALER_gm) | ADC_PRESCALER_DIV16_gc; 



	
	while(true)
	  {				// main loop
	 delay_s(1); 		// delay 1 Sekunde 
	 run_adc();
	 //nop();

	  }
}


void run_adc(void)
{
volatile uint16_t ad_wert; 
   		 

   	ad_wert = get_adValue(); 
 
   	rprintfFloat(3,ad_wert *(1.0/4095.0));		// 3.stellig Float
	//xprintf(PSTR(" ,"));
	 
	xprintf(PSTR("\r")); // neue Zeile

}



/* Read A/D converter NBR_OF_SAMPLES times and return mean value */ 
unsigned int get_adValue(void) 
{ 
   uint16_t test,tmp, sum = 0; 
   uint8_t i;

   for (i=0;i<NBR_OF_SAMPLES;i++)
   	{ 

      /* Start convert on channel0 */ 
      ADCA.CH0.CTRL |= ADC_CH_START_bm; 
       
      do { 
         /* If the conversion on the ADCA channel 0 never is 
          * complete this will be a deadlock. */ 
          test = ADCA.CH0.INTFLAGS; 
      } while (!(test & ADC_CH_CHIF_bm)); 

      /* Clear interrupt flag */ 
      ADCA.CH0.INTFLAGS |= ADC_CH_CHIF_bm; 

      tmp = ADCA.CH0RESL; 
      tmp += ADCA.CH0RESH<<8; 
      sum += tmp; 
     	
	} 

   sum /= NBR_OF_SAMPLES; 
    
   
   PORTB.OUTTGL |= PIN0_bm;  	// Toggle LED   
  
   return  sum; 
}


